@@ -365,6 +365,11 @@ public:
     if (Decls.empty()) {
       if (ResultKind != NotFoundInCurrentInstantiation)
         ResultKind = NotFound;
+
+      if (Paths) {
+        deletePaths(Paths);
+        Paths = 0;
+      }
     } else {
       AmbiguityKind SavedAK = Ambiguity;
       ResultKind = Found;
@@ -434,6 +439,7 @@ public:
     Decls.clear();
     if (Paths) deletePaths(Paths);
     Paths = NULL;
+    NamingClass = 0;
   }
 
   /// \brief Clears out any current state and re-initializes for a
@@ -492,25 +498,18 @@ public:
     LookupResult &Results;
     LookupResult::iterator I;
     bool Changed;
-#ifndef NDEBUG
     bool CalledDone;
-#endif
     
     friend class LookupResult;
     Filter(LookupResult &Results)
-      : Results(Results), I(Results.begin()), Changed(false)
-#ifndef NDEBUG
-      , CalledDone(false)
-#endif
+      : Results(Results), I(Results.begin()), Changed(false), CalledDone(false)
     {}
 
   public:
-#ifndef NDEBUG
     ~Filter() {
       assert(CalledDone &&
              "LookupResult::Filter destroyed without done() call");
     }
-#endif
 
     bool hasNext() const {
       return I != Results.end();
@@ -541,10 +540,8 @@ public:
     }
 
     void done() {
-#ifndef NDEBUG
       assert(!CalledDone && "done() called twice");
       CalledDone = true;
-#endif
 
       if (Changed)
         Results.resolveKindAfterFilter();
@@ -573,11 +570,7 @@ private:
   void configure();
 
   // Sanity checks.
-#ifndef NDEBUG
   void sanity() const;
-#else
-  void sanity() const {}
-#endif
 
   bool sanityCheckUnresolved() const {
     for (iterator I = begin(), E = end(); I != E; ++I)
