@@ -1,9 +1,10 @@
-// RUN: %clang_cc1 -analyze -analyzer-experimental-internal-checks -analyzer-check-objc-mem -analyzer-store=region -analyzer-constraints=range -verify %s
+// RUN: %clang_cc1 -analyze -analyzer-checker=core,core.experimental -analyzer-store=region -analyzer-constraints=range -verify %s
+
 typedef typeof(sizeof(int)) size_t;
 void malloc (size_t);
 
 void f1() {
-  int const &i = 3;
+  int const &i = 3;  // <--- **FIXME** This is currently not being modeled correctly.
   int b = i;
 
   int *p = 0;
