@@ -1,11 +1,11 @@
 // RUN: %clang_cc1 -emit-llvm -o - %s | FileCheck %s
 
 struct Global { Global(); };
-template<typename T> struct X { X(); };
+template<typename T> struct X { X() {} };
 
 
 namespace {
-  struct Anon { Anon(); };
+  struct Anon { Anon() {} };
 
   // CHECK: @_ZN12_GLOBAL__N_15anon0E = internal global
   Global anon0;
