@@ -624,3 +624,26 @@ namespace test20 {
   template <class T> void test1(decltype(f<>(T()))) {}
   template void test1<int>(decltype(f<>(int())));
 }
+
+// rdar:// 8620510
+namespace test21 {
+  // CHECK: define void @_ZN6test2112vla_arg_funcEiPA_i(
+  void vla_arg_func(int X, int a[X][X]) {}
+}
+
+namespace test22 {
+  // CHECK: define void @_ZN6test221fEDn(
+  void f(decltype(nullptr)) { }
+}
+
+// rdar://problem/8913416
+namespace test23 {
+  typedef void * const vpc;
+
+  // CHECK: define void @_ZN6test231fERA10_KPv(
+  void f(vpc (&)[10]) {}
+
+  typedef vpc vpca5[5];
+  void f(vpca5 volatile (&)[10]) {}
+  // CHECK: define void @_ZN6test231fERA10_A5_VKPv(
+}
