@@ -57,3 +57,22 @@ int main() {
   return 0;
 }
 
+// rdar://8379892
+// CHECK: define void @_Z1fP1A
+// CHECK: @objc_msgSend to void
+struct X {
+  X();
+  X(const X&);
+  ~X();
+};
+
+@interface A {
+  X xval;
+}
+- (X)x;
+- (void)setX:(X)x;
+@end
+
+void f(A* a) {
+  a.x = X();
+}
