@@ -30,3 +30,21 @@ void f() {
         id a;
         id b = bar((id <Proto>)a);
 }
+
+// rdar://8472487
+@protocol NSObject @end
+@class NSRunLoop;
+
+@protocol CoreDAVTaskManager <NSObject> 
+  @property (retain) NSRunLoop *workRunLoop;  
+@end
+
+
+// rdar://8475819
+@protocol some_protocol;
+
+void foo (int n)
+{
+  id<some_protocol> array[n];
+}
+
