@@ -14,7 +14,7 @@ short chk1[__alignof__(g3) == 8 ? 1 : -1];
 short chk2[__alignof__(_Complex double) == 8 ? 1 : -1];
 
 // PR6362
-struct __attribute__((packed)) {unsigned int a} g4;
+struct __attribute__((packed)) {unsigned int a;} g4;
 short chk1[__alignof__(g4) == 1 ? 1 : -1];
 short chk2[__alignof__(g4.a) == 1 ? 1 : -1];
 
