@@ -6,13 +6,33 @@ void f(V4i a)
 {
 }
 
-void test()
+void test1()
 {
   V4i vGCC;
   vector int vAltiVec;
 
   f(vAltiVec);
   vGCC = vAltiVec;
-  vGCC = vGCC > vAltiVec;
+  bool res = vGCC > vAltiVec;
   vAltiVec = 0 ? vGCC : vGCC;
 }
+
+template<typename T>
+void template_f(T param) {
+  param++;
+}
+
+void test2()
+{
+  vector int vi;
+  ++vi;
+  vi++;
+  --vi;
+  vi--;
+  vector float vf;
+  vf++;
+
+  ++vi=vi;
+  (++vi)[1]=1;
+  template_f(vi);
+}
