@@ -2,10 +2,10 @@
 
 void f();
 void f(int);
-decltype(f) a; // expected-error{{cannot determine the declared type of an overloaded function}}
+decltype(f) a; // expected-error{{cannot resolve overloaded function from context}}
 
 template<typename T> struct S {
-  decltype(T::f) * f; // expected-error{{cannot determine the declared type of an overloaded function}}
+  decltype(T::f) * f; // expected-error{{cannot resolve overloaded function from context}}
 };
 
 struct K { void f(); void f(int); };
