@@ -16,8 +16,8 @@ void f() {
 }
 
 int a() {
-  const int t=t; // expected-note {{subexpression not valid}}
-  switch(1) {
+  const int t=t;
+  switch(1) { // expected-warning {{no case matching constant switch condition '1'}}
     case t:; // expected-error {{not an integer constant expression}}
   }
 }
