@@ -14,7 +14,7 @@ void test() {
   i = x.operator int();
   x.operator--(); // expected-error{{no member named 'operator--'}}
   x.operator float(); // expected-error{{no member named 'operator float'}}
-  x.operator; // expected-error{{missing type specifier after 'operator'}}
+  x.operator; // expected-error{{expected a type}}
 }
 
 void test2() {
@@ -25,5 +25,5 @@ void test2() {
   i = x->operator int();
   x->operator--(); // expected-error{{no member named 'operator--'}}
   x->operator float(); // expected-error{{no member named 'operator float'}}
-  x->operator; // expected-error{{missing type specifier after 'operator'}}
+  x->operator; // expected-error{{expected a type}}
 }
