@@ -32,17 +32,6 @@ namespace PR5531 {
   }
 }
 
-
-struct X {
- int foo() __attribute__((warn_unused_result));
-};
-
-void bah() {
-  X x, *x2;
-  x.foo(); // expected-warning {{ignoring return value of function declared with warn_unused_result attribute}}
-  x2->foo(); // expected-warning {{ignoring return value of function declared with warn_unused_result attribute}}
-}
-
 template<typename T>
 struct X0 { };
 
@@ -59,3 +48,9 @@ namespace PR6948 {
     X<char> str (read_from_file()); // expected-error{{use of undeclared identifier 'read_from_file'}}
   }
 }
+
+void unused_local_static() {
+  static int x = 0;
+  static int y = 0; // expected-warning{{unused variable 'y'}}
+#pragma unused(x)
+}
